export declare type FieldDescriptorParams = Partial<Pick<FieldDescriptor, keyof FieldDescriptor>>;
export declare type ModelDescriptorParams = Partial<Pick<ModelDescriptor, keyof ModelDescriptor>>;
export declare class ModelDescriptor {
    name: string;
    label: string;
    plural: string;
    description: string;
    token: string;
    tokens: string;
    fields: FieldDescriptorSet;
    constructor(name?: string, params?: Partial<Pick<ModelDescriptor, keyof ModelDescriptor>>);
    constructor(params?: Partial<Pick<ModelDescriptor, keyof ModelDescriptor>>);
    field(name: string): FieldDescriptor;
    add(field: FieldDescriptor): void;
    autopopulate(): void;
}
export declare class ViewDescriptor extends ModelDescriptor {
    title?: string;
    model?: ModelDescriptor;
    constructor(model?: ModelDescriptor);
    constructor(name?: string, params?: Partial<Pick<ModelDescriptor, keyof ModelDescriptor>>);
    constructor(params?: Partial<Pick<ModelDescriptor, keyof ModelDescriptor>>);
}
export declare class FieldDescriptor {
    name?: string;
    label?: string;
    plural?: string;
    description?: string;
    length?: number;
    link?: string;
    column?: string;
    required?: boolean;
    token?: string;
    tokens?: string;
    type?: string;
    widget?: string;
    constructor();
    constructor(name: string, type?: string, widget?: string);
    constructor(params: FieldDescriptorParams);
    autopopulate(): void;
}
/**
 * A set containing the managed properties that exist on an object.
 */
export declare class FieldDescriptorSet {
    private ʘ;
    constructor(from?: FieldDescriptorSet);
    get length(): number;
    /**
     * Merge descriptors from another set into this set
     * @param from
     */
    merge(from: FieldDescriptorSet): void;
    /**
     * Does a descriptor with the given name exist in the set
     * @param name
     */
    has(name: string): boolean;
    /**
     * Get the descriptor with the provided name
     * @param name
     */
    get(name: string): FieldDescriptor;
    /**
     * Get names of all descriptors which exist in the set.
     */
    get names(): Array<string>;
    /**
     * Set the descriptor of the given name
     * @param name
     * @param descriptor
     */
    set(name: string, descriptor: FieldDescriptor): FieldDescriptor;
    /**
     * Execute a function on each item in the set
     * @param callback
     */
    forEach(callback: (propertyName: string, definition: FieldDescriptor) => void): void;
}
