"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const descriptors_1 = require("../descriptors");
const field_1 = require("./field");
const model_1 = require("./model");
const view_1 = require("./view");
describe('View', () => {
    it('should create a model descriptor', () => {
        let Foo = class Foo {
        };
        __decorate([
            field_1.Field,
            __metadata("design:type", String)
        ], Foo.prototype, "bar", void 0);
        Foo = __decorate([
            model_1.Model
        ], Foo);
        const d = Reflect.getMetadata("model:descriptor", Foo);
        expect(d).toBeInstanceOf(descriptors_1.ModelDescriptor);
        expect(d.name).toBe('Foo');
    });
    it('should create a view descriptor', () => {
        let Foo = class Foo {
        };
        __decorate([
            field_1.Field,
            __metadata("design:type", String)
        ], Foo.prototype, "bar", void 0);
        Foo = __decorate([
            model_1.Model
        ], Foo);
        ;
        let FooDetail = class FooDetail {
        };
        FooDetail = __decorate([
            (0, view_1.View)(Foo)
        ], FooDetail);
        const d = Reflect.getMetadata("model:descriptor", FooDetail);
        expect(d).toBeInstanceOf(descriptors_1.ViewDescriptor);
    });
    it('should have the field from the model', () => {
        let Foo = class Foo {
        };
        __decorate([
            field_1.Field,
            __metadata("design:type", String)
        ], Foo.prototype, "bar", void 0);
        Foo = __decorate([
            model_1.Model
        ], Foo);
        ;
        let FooDetail = class FooDetail {
        };
        FooDetail = __decorate([
            (0, view_1.View)(Foo)
        ], FooDetail);
        const d = Reflect.getMetadata("model:descriptor", FooDetail);
        expect(d.fields.length).toBe(1);
        expect(d.fields.has('bar')).toBeTrue();
    });
    it('should have the fields from the model', () => {
        let Foo = class Foo {
        };
        __decorate([
            field_1.Field,
            __metadata("design:type", String)
        ], Foo.prototype, "bar", void 0);
        __decorate([
            field_1.Field,
            __metadata("design:type", Number)
        ], Foo.prototype, "biz", void 0);
        Foo = __decorate([
            model_1.Model
        ], Foo);
        ;
        let FooDetail = class FooDetail {
        };
        FooDetail = __decorate([
            (0, view_1.View)(Foo)
        ], FooDetail);
        const d = Reflect.getMetadata("model:descriptor", FooDetail);
        expect(d.fields.length).toBe(2);
        expect(d.fields.has('bar')).toBeTrue();
        expect(d.fields.has('biz')).toBeTrue();
    });
    it('should have no fields from the model', () => {
        let Foo = class Foo {
        };
        __decorate([
            field_1.Field,
            __metadata("design:type", String)
        ], Foo.prototype, "bar", void 0);
        __decorate([
            field_1.Field,
            __metadata("design:type", Number)
        ], Foo.prototype, "biz", void 0);
        Foo = __decorate([
            model_1.Model
        ], Foo);
        let FooNone = class FooNone {
        };
        FooNone = __decorate([
            (0, view_1.View)(Foo, [])
        ], FooNone);
        const d = Reflect.getMetadata("model:descriptor", FooNone);
        expect(d.fields.length).toBe(0);
    });
    fit('should have the specified field from the model', () => {
        let Foo = class Foo {
        };
        __decorate([
            field_1.Field,
            __metadata("design:type", String)
        ], Foo.prototype, "bar", void 0);
        __decorate([
            field_1.Field,
            __metadata("design:type", Number)
        ], Foo.prototype, "biz", void 0);
        Foo = __decorate([
            model_1.Model
        ], Foo);
        let FooNone = class FooNone {
        };
        FooNone = __decorate([
            (0, view_1.View)(Foo, ['bar'])
        ], FooNone);
        const d = Reflect.getMetadata("model:descriptor", FooNone);
        expect(d.fields.has('bar')).toBeTrue();
        expect(d.fields.has('biz')).toBeFalse();
    });
    fit('should throw an error if the field does not exist on the model', () => {
        let Foo = class Foo {
        };
        __decorate([
            field_1.Field,
            __metadata("design:type", String)
        ], Foo.prototype, "bar", void 0);
        __decorate([
            field_1.Field,
            __metadata("design:type", Number)
        ], Foo.prototype, "biz", void 0);
        Foo = __decorate([
            model_1.Model
        ], Foo);
        expect(() => {
            // @ts-ignore (correctly failes typescript checks)
            let FooNone = class FooNone {
            };
            FooNone = __decorate([
                (0, view_1.View)(Foo, ['buz'])
            ], FooNone);
        }).toThrowError();
    });
    describe('field selection parameter', () => {
        describe('pick', () => {
            it('should add the specified fields', () => {
            });
            it('should not add the other fields', () => {
            });
            it('cannot be used with omit', () => {
            });
            it('cannot be used with partial', () => {
            });
        });
        describe('omit', () => {
            it('should not add the specified fields', () => {
            });
            it('should add the other fields', () => {
            });
            it('cannot be used with pick', () => {
            });
            it('cannot be used with partial', () => {
            });
        });
        describe('partial', () => {
            it('should add all fields', () => {
            });
            it('should set all fields required to false', () => {
            });
            it('accepts a boolean', () => {
            });
            it('cannot be used with pick', () => {
            });
            it('cannot be used with omit', () => {
            });
            it('cannot be used with optional', () => {
            });
            it('cannot be used with required', () => {
            });
        });
        describe('optional', () => {
            it('add the specified fields', () => {
            });
            it('sets the specified fields required to false', () => {
            });
            it('does not mix fields with required', () => {
            });
        });
        describe('required', () => {
            it('add the specified fields', () => {
            });
            it('sets the specified fields required to true', () => {
            });
            it('does not mix fields with optional', () => {
            });
        });
    });
});
