"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.coerce = void 0;
const meta_1 = require("../meta");
require("reflect-metadata");
function coerce(to, ...args) {
    function coerce(target, name, descriptor) {
        if (descriptor)
            throw new Error("Cannot use the coerce decorator on a method");
        /* if the to type wasn't specified try getting it from the reflect meta
           data */
        if (to === undefined) {
            const valueType = Reflect.getMetadata("design:type", target, name);
            if (valueType === Array) {
                throw new Error(`Sorry, in order to use @coerce on an array
                property you must explicitly pass in the element type as
                a parameter. See https://github.com/microsoft/TypeScript/issues/7169
                and... leave a comment?
                
                @coerce([MyClass]) ${name}`);
            }
            to = valueType;
        }
        (0, meta_1.meta)(target).property(name).coerce(to);
    }
    if (args.length > 1) {
        let [target, name, descriptor] = [to, ...args];
        to = undefined;
        return coerce(target, name, descriptor);
    }
    else {
        return coerce;
    }
}
exports.coerce = coerce;
