"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PropertyDescriptorSet = exports.PropertyDescriptor = exports.ObjectDescriptor = exports.MethodDescriptorSet = exports.MethodDescriptor = void 0;
/**
 * Describes a method and all associated modifiers and provides the
 * method dispatcher.
 */
class MethodDescriptor {
    /**
     * Add a method to be called after the primary method
     * @param value Function or method to be called
     */
    after(value) {
        this.ʘafter || (this.ʘafter = []);
        this.ʘafter.push(value);
        return this;
    }
    // around( value: (original, ...args) => any ) {
    //     let original = this.ʘvalue
    //     this.ʘvalue  =  function ( ...args:any[] ) {
    //         return value.call(this, original, ...args )
    //     }
    //     return this
    // }
    /**
     * Add a method to be called before the primary method
     * @param value Function or method to be called
     */
    before(value) {
        this.ʘbefore || (this.ʘbefore = []);
        this.ʘbefore.push(value);
        return this;
    }
    /**
     * Set the default value for the primary method. This is the primary
     * method which will be called, unless it has been over-ridden.
     * @param value
     */
    default(value) {
        this.ʘdefault = value;
        return this;
    }
    /**
     * Set the over-ride value for the primary method. This will be
     * called instead of the default value.
     */
    override(value) {
        this.ʘoverride = value;
        return this;
    }
    /**
     * Add a method to be called after the primary method but before
     * any after methods
     * @param value
     */
    stack(value) {
        this.ʘstack || (this.ʘstack = []);
        this.ʘstack.push(value);
        return this;
    }
    /**
     * Has a modified been set on the descriptor, returns true or false.
     * @param modifier
     */
    does(modifier) {
        return this[`ʘ${modifier}`] ? true : false;
    }
    /**
     * Execute the method and all method modifiers
     * @param target The object to call the method on
     * @param args The arguments to pass to the object
     */
    call(target, ...args) {
        this.ʘbefore && this.callStack('before', target, ...args);
        /* call the 'primary' function */
        let f = this.ʘoverride || this.ʘdefault || function () { };
        let r = f.call(target, ...args);
        this.ʘstack && this.callStack('stack', target, ...args);
        this.ʘafter && this.callStack('after', target, ...args);
        return r;
    }
    /**
     * Call a stack of methods
     * @param modifier The name of the stack to execute
     * @param target The object on which to act
     * @param args The arguments to pass
     */
    callStack(modifier, target, ...args) {
        for (let func of this['ʘ' + modifier]) {
            func.call(target, ...args);
        }
    }
    /**
     * Include the definitions from another method descriptor into this descriptor,
     * used to merge descriptors when applying traits to a class
     * @param from Descriptor to include
     */
    include(from) {
        if (this.ʘdefault === undefined)
            this.ʘdefault = from.ʘdefault;
        if (!(from.ʘoverride === undefined))
            this.ʘoverride = from.ʘoverride;
        if (from.ʘafter) {
            this.ʘafter || (this.ʘafter = []);
            this.ʘafter.push(...from.ʘafter);
        }
        if (from.ʘbefore) {
            this.ʘbefore || (this.ʘbefore = []);
            this.ʘbefore.push(...from.ʘbefore);
        }
        if (from.ʘstack) {
            this.ʘstack || (this.ʘstack = []);
            this.ʘstack.push(...from.ʘstack);
        }
    }
    /**
     * Install the method dispatcher which handles calling of any modifiers
     * into the target object
     * @param target Target object to install the dispatcher
     * @param name Name of the method to replace
     */
    install_dispatcher(target, name) {
        var _a, _b;
        let descriptor = this;
        /* store the "original" method as the default implementation
           before installing the dispatcher */
        let original = (_a = Object.getOwnPropertyDescriptor(target, name)) === null || _a === void 0 ? void 0 : _a.value;
        if (original) {
            this.ʘdefault = original;
        }
        /* if there is no original method defined on the target, but the
        method does exist in the targets prototype heirarchy, then mark
        this dispatcher as transient, and merge it with the ancestral
        dispatcher */
        else if (target[name] !== undefined) {
            this.ʘtransient = true;
            let cursor = Object.getPrototypeOf(target);
            while (cursor) {
                if (cursor.hasOwnProperty(name)) {
                    if ((_b = cursor.Δmeta) === null || _b === void 0 ? void 0 : _b.methods.has(name)) {
                        this.include(cursor.Δmeta.methods.get(name));
                    }
                    else {
                        this.ʘdefault = Object.getOwnPropertyDescriptor(cursor, name).value;
                    }
                    cursor = null;
                }
                else {
                    cursor = Object.getPrototypeOf(cursor);
                }
            }
        }
        Object.defineProperty(target, name, {
            value: function (...args) {
                return descriptor.call(this, ...args);
            },
            writable: true,
            configurable: true
        });
    }
}
exports.MethodDescriptor = MethodDescriptor;
/**
 * A set containing the managed methods that exist on an object.
 */
class MethodDescriptorSet {
    constructor(from) {
        this.ʘ = {};
        if (from)
            this.merge(from);
    }
    /**
     * Merge descriptors from another set into this set
     * @param from
     */
    merge(from) {
        Object.assign(this.ʘ, from.ʘ);
    }
    /**
     * Does a descriptor with the given name exist in the set
     * @param name
     */
    has(name) {
        return name in this.ʘ;
    }
    /**
     * Get a descriptor for the given name, creates a descriptor if
     * one does not exist.
     * @param name
     */
    get(name) {
        if (!this.has(name))
            this.ʘ[name] = new MethodDescriptor();
        return this.ʘ[name];
    }
    /**
     * Get names of all descriptors which exist in the set.
     */
    get names() {
        return Object.getOwnPropertyNames(this.ʘ);
    }
    /**
     * Set the descriptor for the given name
     * @param name
     * @param descriptor
     */
    set(name, descriptor) {
        return this.ʘ[name] = descriptor;
    }
    /**
     * Execute a function on each item in the set
     * @param callback
     */
    forEach(callback) {
        for (let methodName in this.ʘ) {
            callback(methodName, this.ʘ[methodName]);
        }
    }
}
exports.MethodDescriptorSet = MethodDescriptorSet;
/**
 * Describes an object, holding information about methods, properties, and traits.
 */
class ObjectDescriptor {
    constructor(target) {
        this.target = target;
        this.properties = new PropertyDescriptorSet(this);
        this.methods = new MethodDescriptorSet();
    }
    does(trait) {
        var _a, _b;
        // if ( typeof trait === "function" ) trait = trait.prototype
        if ((_a = this.traits) === null || _a === void 0 ? void 0 : _a.includes(trait))
            return true;
        let iter = this.target;
        while (iter = Object.getPrototypeOf(iter)) {
            if (iter === Object)
                return false;
            if ((_b = iter.Δmeta) === null || _b === void 0 ? void 0 : _b.does(trait))
                return true;
        }
        return false;
    }
    // public get super( ):ObjectDescriptor {
    //     let parent = Object.getPrototypeOf(this.target)
    //     if ( ! parent ) return
    //     if ( ! parent.hasOwnProperty('Δmeta') ) {
    //         Object.defineProperty(parent, 'Δmeta', {
    //             value: new ObjectDescriptor(parent),
    //             writable: false,
    //             enumerable: false
    //         })
    //     }
    //     return parent['Δmeta']
    // }
    /**
     * Returns a PropertyDescriptor for the property with the given name. If
     * a descriptor does not exist for the property, one will be created for it
     * and the dispatcher will be installed to the object.
     * @param name R
     */
    property(name) {
        let descriptor;
        if (!this.properties.has(name)) {
            descriptor = new PropertyDescriptor(this, name);
            this.properties.set(name, descriptor);
            descriptor.install_dispatcher();
        }
        else {
            descriptor = this.properties.get(name);
        }
        return descriptor;
    }
    /**
     * Returns a MethodDescriptor the method with the given name. If a desriptor
     * does not exist for the method, one will be created for it and the dispatcher
     * will be created for it and the dispatcher will be installed to the object.
     */
    method(name) {
        let descriptor;
        if (!this.methods.has(name)) {
            descriptor = new MethodDescriptor();
            this.methods.set(name, descriptor);
            descriptor.install_dispatcher(this.target, name);
        }
        else {
            descriptor = this.methods.get(name);
        }
        return descriptor;
    }
    /**
    * Include traits the specified traits into the ObjectDescriptor, copying
    * over default method definitions and merging all property and method
    * descriptors as applicable.
    * @param traits
    */
    include(...traits) {
        var _a, _b, _c, _d, _e, _f, _g;
        this.traits || (this.traits = []);
        let targetConstructor, target;
        if (typeof this.target === "function") {
            targetConstructor = this.target;
            target = this.target.prototype;
        }
        else {
            target = this.target;
            targetConstructor = this.target.constructor;
        }
        this.traits.push(...traits);
        for (let trait of traits) {
            if (typeof trait === "function")
                trait = trait.prototype;
            /* copy traits which have been applied to the incoming trait in to
               the applied traits tracking array */
            if ((_a = trait.Δmeta) === null || _a === void 0 ? void 0 : _a.traits)
                this.traits.push(...trait.Δmeta.traits);
            /* copy over default method implementations */
            for (let propertyName of Object.getOwnPropertyNames(trait)) {
                if (propertyName[0] === "ʘ")
                    continue;
                if (propertyName[0] === "Δ")
                    continue;
                if (propertyName === "constructor")
                    continue;
                /* don't over-write existing definitions from the consuming class */
                if ((_b = target.Δmeta) === null || _b === void 0 ? void 0 : _b.methods.has(propertyName))
                    continue;
                if ((_c = target.Δmeta) === null || _c === void 0 ? void 0 : _c.properties.has(propertyName))
                    continue;
                /* don't copy anything that is managed by agape meta data */
                if ((_d = trait.Δmeta) === null || _d === void 0 ? void 0 : _d.methods.has(propertyName))
                    continue;
                if ((_e = trait.Δmeta) === null || _e === void 0 ? void 0 : _e.properties.has(propertyName))
                    continue;
                /* don't copy over anything that is already defined in the target */
                if (propertyName in target)
                    continue;
                Object.defineProperty(target, propertyName, Object.assign({}, Object.getOwnPropertyDescriptor(trait, propertyName)));
            }
            /* copy over managed methods */
            if ((_f = trait.Δmeta) === null || _f === void 0 ? void 0 : _f.methods) {
                for (let name of trait.Δmeta.methods.names) {
                    if (!target.Δmeta.methods.has(name)) {
                        target.Δmeta.method(name).install_dispatcher(target);
                    }
                    target.Δmeta.method(name).include(trait.Δmeta.method(name));
                }
            }
            /* copy over managed properties */
            if ((_g = trait.Δmeta) === null || _g === void 0 ? void 0 : _g.properties) {
                for (let name of trait.Δmeta.properties.names) {
                    if (!target.Δmeta.properties.has(name)) {
                        target.Δmeta.property(name).install_dispatcher(target);
                    }
                    target.Δmeta.property(name).include(trait.Δmeta.property(name));
                }
            }
            /* apply Δdecorator */
            if (trait.Δdecorate) {
                targetConstructor = trait.Δdecorate(targetConstructor);
                target = targetConstructor.prototype;
            }
            /* apply Δinclude */
            if (trait.Δinclude) {
                trait.Δinclude(target, trait);
            }
        }
        return targetConstructor;
    }
}
exports.ObjectDescriptor = ObjectDescriptor;
/**
 * Describes the property of an object any associated modifiers. Provides
 * the property dispatcher.
 */
class PropertyDescriptor {
    /**
     * @param progenitor The object to which the property belongs
     * @param name The name of the property
     */
    constructor(progenitor, name) {
        this.progenitor = progenitor;
        this.name = name;
    }
    /**
     * Declare the type to coerce to when inflating objects
     * @param to The class to instantiate with the data
     */
    coerce(to) {
        this.ʘcoerce = to;
        return this;
    }
    /**
     * Delegate getting and setting of the property to that of another object
     * @param to The object to delegate the property to
     * @param property The name of the property to delegate to
     */
    delegate(to, property) {
        this.ʘdelegate || (this.ʘdelegate = {});
        this.ʘdelegate.to = to;
        if (property != undefined)
            this.ʘdelegate.property = property;
        return this;
    }
    /**
     * Set default value for the property, can be a primitive value or a callback
     * function which returns any value type, such as a data structure or object
     * @param value Default value
     */
    default(value) {
        this['ʘdefault'] === undefined && (this['ʘdefault'] = value);
        return this;
    }
    /**
     * By setting enumerable to false the property will not be included when
     * iterating over the properties of the object. The property will also
     * not be included when printing using console.log()
     * @param value True or false
     */
    enumerable(value) {
        if (this.ʘenumerable != value) {
            this.ʘenumerable = value;
            this.install_dispatcher();
        }
        return this;
    }
    /**
     * Readonly properties will throw an exception when attempting to set the value
     * @param value True or false
     */
    readonly(value = true) {
        this.ʘreadonly = value;
        return this;
    }
    /**
     * Override the default value of the property
     * @param value
     */
    override(value) {
        this.ʘoverride = value;
        return this;
    }
    /**
     * Include the definitions from another property descriptor into this descriptor,
     * used to merge descriptors when applying traits to a class
     * @param from Descriptor to include
     */
    include(from) {
        if (this.ʘdefault === undefined || from.ʘoverride === true)
            this.ʘdefault = from.ʘdefault;
        if (!(from.ʘoverride === undefined))
            this.ʘoverride = from.ʘoverride;
        if (!(from.ʘreadonly === undefined))
            this.ʘreadonly = from.ʘreadonly;
        return this;
    }
    /**
     * Inherit the value from another object
     * @param from
     * @param property
     */
    inherit(from, property) {
        this.ʘinherit || (this.ʘinherit = {});
        this.ʘinherit.from = from;
        if (property != undefined)
            this.ʘinherit.property = property;
        return this;
    }
    /**
     * Get the value of the property on the given object, delegating or building
     * the property value as necessary
     * @param instance The object on which to act
     */
    get(instance) {
        /* delegate */
        if (this.ʘdelegate && this.ʘdelegate.to) {
            return typeof this.ʘdelegate.to === "function"
                ? this.ʘdelegate.to.call(instance, instance)[this.ʘdelegate.property || this.name]
                : this.ʘdelegate.to[this.ʘdelegate.property || this.name];
        }
        if (!instance['ʘ' + this.name]) {
            /* inherit */
            if (this.ʘinherit) {
                if (typeof this.ʘinherit.from === "function") {
                    let from = this.ʘinherit.from.call(instance, instance);
                    return from !== undefined ? from[this.ʘinherit.property || this.name] : undefined;
                }
                else {
                    return this.ʘinherit.from ? this.ʘinherit.from[this.ʘinherit.property || this.name] : undefined;
                }
            }
            /* default (lazy) */
            let value = typeof this.ʘdefault === "function"
                ? this.ʘdefault.call(instance, instance)
                : this.ʘdefault;
            Object.defineProperty(instance, `ʘ${this.name}`, { value: value, configurable: true, enumerable: false });
        }
        return instance['ʘ' + this.name];
    }
    /**
     * Set the value of the property on the given instance, delegating if needed
     * @param instance The object on which to act
     * @param value The new value
     */
    set(instance, value) {
        /* read only */
        if (this.ʘreadonly)
            throw new Error('Cannot write to read-only value');
        /* delegate */
        if (this.ʘdelegate && this.ʘdelegate.to) {
            return typeof this.ʘdelegate.to === "function"
                ? this.ʘdelegate.to.call(instance, instance)[this.ʘdelegate.property || this.name] = value
                : this.ʘdelegate.to[this.ʘdelegate.property || this.name] = value;
        }
        Object.defineProperty(instance, `ʘ${this.name}`, { value: value, configurable: true, enumerable: false });
    }
    /**
     * Install the property dispatcher into the target object. The dispatcher handles
     * any property modifiers which have been applied.
     * @param target Target object to install the dispatcher
     * @param name Name of the method to replace
     */
    install_dispatcher() {
        let descriptor = this;
        let target = this.progenitor.target;
        Object.defineProperty(target, descriptor.name, {
            set: function (value) { return descriptor.set(this, value); },
            get: function () { return descriptor.get(this); },
            enumerable: this.ʘenumerable === false ? false : true,
            configurable: true
        });
        Object.defineProperty(target, 'ʘ' + descriptor.name, {
            enumerable: false,
            configurable: true,
            writable: true
        });
    }
}
exports.PropertyDescriptor = PropertyDescriptor;
/**
 * A set containing the managed properties that exist on an object.
 */
class PropertyDescriptorSet {
    constructor(progenitor, from) {
        this.progenitor = progenitor;
        this.ʘ = {};
        if (from)
            this.merge(from);
    }
    /**
     * Merge descriptors from another set into this set
     * @param from
     */
    merge(from) {
        Object.assign(this.ʘ, from.ʘ);
    }
    /**
     * Does a descriptor with the given name exist in the set
     * @param name
     */
    has(name) {
        return name in this.ʘ;
    }
    /**
     * Get a descriptor for the given name, creates a descriptor if
     * one does not exist.
     * @param name
     */
    get(name) {
        if (!this.has(name))
            this.ʘ[name] = new PropertyDescriptor(this.progenitor, name);
        return this.ʘ[name];
    }
    /**
     * Get names of all descriptors which exist in the set.
     */
    get names() {
        return Object.getOwnPropertyNames(this.ʘ);
    }
    /**
     * Set the descriptor of the given name
     * @param name
     * @param descriptor
     */
    set(name, descriptor) {
        return this.ʘ[name] = descriptor;
    }
    /**
     * Execute a function on each item in the set
     * @param callback
     */
    forEach(callback) {
        for (let propertyName in this.ʘ) {
            callback(propertyName, this.ʘ[propertyName]);
        }
    }
}
exports.PropertyDescriptorSet = PropertyDescriptorSet;
