"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deflate = exports.inflateArray = exports.inflateObject = exports.inflate = exports.Serializer = void 0;
const meta_1 = require("./meta");
class Serializer {
    constructor(to) {
        this.to = to;
    }
    inflate(data) {
        return inflate(this.to, data);
    }
    deflate(item, params) {
        return deflate(item, params);
    }
}
exports.Serializer = Serializer;
function inflate(to, from) {
    return Array.isArray(to)
        ? inflateArray(to[0], from)
        : inflateObject(to, from);
}
exports.inflate = inflate;
function inflateObject(to, from) {
    if (to instanceof Serializer) {
        return to.inflate(from);
    }
    else {
        const o = new to();
        const m = (0, meta_1.meta)(to);
        for (let key in from) {
            if (m.properties.has(key) && m.property(key).ʘcoerce) {
                const coerce = m.property(key).ʘcoerce;
                o[key] = inflate(coerce, from[key]);
            }
            else {
                o[key] = from[key];
            }
        }
        return o;
    }
}
exports.inflateObject = inflateObject;
function inflateArray(to, from) {
    return from.map(e => inflateObject(to, e));
}
exports.inflateArray = inflateArray;
/**
 * Deflate
 */
function deflate(item, params) {
    if (item instanceof Object) {
        return deflateObject(item);
    }
    else if (item instanceof Array) {
        return deflateArray(item);
    }
    else
        return item;
}
exports.deflate = deflate;
function deflateObject(item, params) {
    let r = {};
    let m = 'Δmeta' in item ? item['Δmeta'] : null;
    for (let field in item) {
        /* if this is property is managed by agape */
        if (m && m.properties.has(field)) {
            /* ignore delegated properties */
            if (m.property(field)['ʘdelegate'])
                continue;
            /* ignore unpopulated inheritied properties with undefined value */
            if (m.property(field)['ʘinherit']) {
                if (item[`ʘ${field}`] === undefined) {
                    continue;
                }
            }
            if (m.property(field)['ʘcoerce']) {
                const coerce = m.property(field)['ʘcoerce'];
                if (coerce instanceof Serializer) {
                    r[field] = coerce.deflate(item[field]);
                }
                else if (Array.isArray(coerce) && coerce[0] instanceof Serializer) {
                    r[field] = item[field].map(value => coerce[0].deflate(value));
                }
                else {
                    r[field] = deflate(item[field]);
                }
                continue;
            }
        }
        let value = item[field];
        /* get the deflated value, skip method definitions, this is required when exporting
        to ES5, because method definitions show up in for .. in, however as of ES6 and later
        methods are consider enumerable. We will check if the value is a function here, if it
        is, the field will not be serialized */
        if (typeof value != "function")
            r[field] = deflate(value);
    }
    return r;
}
function deflateArray(items, params) {
    return items.map(item => deflate(item));
}
